# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from the asn1ate tool.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# PKCS #12: Personal Information Exchange Syntax v1.1
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7292.txt
# https://www.rfc-editor.org/errata_search.php?rfc=7292

from pyasn1.type import char, constraint, namedtype, namedval, opentype, tag, univ

from pyasn1_modules import rfc2315, rfc5280, rfc5652, rfc5958


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


# Initialize the maps used in PKCS#12

pkcs12BagTypeMap = {}

pkcs12CertBagMap = {}

pkcs12CRLBagMap = {}

pkcs12SecretBagMap = {}


# Imports from RFC 2315, RFC 5652, and RFC 5958

DigestInfo = rfc2315.DigestInfo


ContentInfo = rfc5652.ContentInfo

PKCS12Attribute = rfc5652.Attribute


EncryptedPrivateKeyInfo = rfc5958.EncryptedPrivateKeyInfo

PrivateKeyInfo = rfc5958.PrivateKeyInfo


# CMSSingleAttribute is the same as Attribute in RFC 5652 except the attrValues
# SET must have one and only one member


class AttributeType(univ.ObjectIdentifier):
    pass


class AttributeValue(univ.Any):
    pass


class AttributeValues(univ.SetOf):
    pass


AttributeValues.componentType = AttributeValue()


class CMSSingleAttribute(univ.Sequence):
    pass


CMSSingleAttribute.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", AttributeType()),
    namedtype.NamedType(
        "attrValues",
        AttributeValues().subtype(sizeSpec=constraint.ValueSizeConstraint(1, 1)),
        openType=opentype.OpenType("attrType", rfc5652.cmsAttributesMap),
    ),
)


# Object identifier arcs

rsadsi = _OID(1, 2, 840, 113549)

pkcs = _OID(rsadsi, 1)

pkcs_9 = _OID(pkcs, 9)

certTypes = _OID(pkcs_9, 22)

crlTypes = _OID(pkcs_9, 23)

pkcs_12 = _OID(pkcs, 12)


# PBE Algorithm Identifiers and Parameters Structure

pkcs_12PbeIds = _OID(pkcs_12, 1)

pbeWithSHAAnd128BitRC4 = _OID(pkcs_12PbeIds, 1)

pbeWithSHAAnd40BitRC4 = _OID(pkcs_12PbeIds, 2)

pbeWithSHAAnd3_KeyTripleDES_CBC = _OID(pkcs_12PbeIds, 3)

pbeWithSHAAnd2_KeyTripleDES_CBC = _OID(pkcs_12PbeIds, 4)

pbeWithSHAAnd128BitRC2_CBC = _OID(pkcs_12PbeIds, 5)

pbeWithSHAAnd40BitRC2_CBC = _OID(pkcs_12PbeIds, 6)


class Pkcs_12PbeParams(univ.Sequence):
    pass


Pkcs_12PbeParams.componentType = namedtype.NamedTypes(
    namedtype.NamedType("salt", univ.OctetString()),
    namedtype.NamedType("iterations", univ.Integer()),
)


# Bag types

bagtypes = _OID(pkcs_12, 10, 1)


class BAG_TYPE(univ.Sequence):
    pass


BAG_TYPE.componentType = namedtype.NamedTypes(
    namedtype.NamedType("id", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "unnamed1", univ.Any(), openType=opentype.OpenType("attrType", pkcs12BagTypeMap)
    ),
)


id_keyBag = _OID(bagtypes, 1)


class KeyBag(PrivateKeyInfo):
    pass


id_pkcs8ShroudedKeyBag = _OID(bagtypes, 2)


class PKCS8ShroudedKeyBag(EncryptedPrivateKeyInfo):
    pass


id_certBag = _OID(bagtypes, 3)


class CertBag(univ.Sequence):
    pass


CertBag.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certId", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "certValue",
        univ.Any().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
        openType=opentype.OpenType("certId", pkcs12CertBagMap),
    ),
)


x509Certificate = CertBag()
x509Certificate["certId"] = _OID(certTypes, 1)
x509Certificate["certValue"] = univ.OctetString()
# DER-encoded X.509 certificate stored in OCTET STRING


sdsiCertificate = CertBag()
sdsiCertificate["certId"] = _OID(certTypes, 2)
sdsiCertificate["certValue"] = char.IA5String()
# Base64-encoded SDSI certificate stored in IA5String


id_CRLBag = _OID(bagtypes, 4)


class CRLBag(univ.Sequence):
    pass


CRLBag.componentType = namedtype.NamedTypes(
    namedtype.NamedType("crlId", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "crlValue",
        univ.Any().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
        openType=opentype.OpenType("crlId", pkcs12CRLBagMap),
    ),
)


x509CRL = CRLBag()
x509CRL["crlId"] = _OID(crlTypes, 1)
x509CRL["crlValue"] = univ.OctetString()
# DER-encoded X.509 CRL stored in OCTET STRING


id_secretBag = _OID(bagtypes, 5)


class SecretBag(univ.Sequence):
    pass


SecretBag.componentType = namedtype.NamedTypes(
    namedtype.NamedType("secretTypeId", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "secretValue",
        univ.Any().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
        openType=opentype.OpenType("secretTypeId", pkcs12SecretBagMap),
    ),
)


id_safeContentsBag = _OID(bagtypes, 6)


class SafeBag(univ.Sequence):
    pass


SafeBag.componentType = namedtype.NamedTypes(
    namedtype.NamedType("bagId", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "bagValue",
        univ.Any().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
        openType=opentype.OpenType("bagId", pkcs12BagTypeMap),
    ),
    namedtype.OptionalNamedType(
        "bagAttributes", univ.SetOf(componentType=PKCS12Attribute())
    ),
)


class SafeContents(univ.SequenceOf):
    pass


SafeContents.componentType = SafeBag()


# The PFX PDU


class AuthenticatedSafe(univ.SequenceOf):
    pass


AuthenticatedSafe.componentType = ContentInfo()
# Data if unencrypted
# EncryptedData if password-encrypted
# EnvelopedData if public key-encrypted


class MacData(univ.Sequence):
    pass


MacData.componentType = namedtype.NamedTypes(
    namedtype.NamedType("mac", DigestInfo()),
    namedtype.NamedType("macSalt", univ.OctetString()),
    namedtype.DefaultedNamedType("iterations", univ.Integer().subtype(value=1))
    # Note: The default is for historical reasons and its use is deprecated
)


class PFX(univ.Sequence):
    pass


PFX.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "version", univ.Integer(namedValues=namedval.NamedValues(("v3", 3)))
    ),
    namedtype.NamedType("authSafe", ContentInfo()),
    namedtype.OptionalNamedType("macData", MacData()),
)


# Local key identifier (also defined as certificateAttribute in rfc2985.py)

pkcs_9_at_localKeyId = _OID(pkcs_9, 21)

localKeyId = CMSSingleAttribute()
localKeyId["attrType"] = pkcs_9_at_localKeyId
localKeyId["attrValues"][0] = univ.OctetString()


# Friendly name (also defined as certificateAttribute in rfc2985.py)

pkcs_9_ub_pkcs9String = univ.Integer(255)

pkcs_9_ub_friendlyName = univ.Integer(pkcs_9_ub_pkcs9String)

pkcs_9_at_friendlyName = _OID(pkcs_9, 20)


class FriendlyName(char.BMPString):
    pass


FriendlyName.subtypeSpec = constraint.ValueSizeConstraint(1, pkcs_9_ub_friendlyName)


friendlyName = CMSSingleAttribute()
friendlyName["attrType"] = pkcs_9_at_friendlyName
friendlyName["attrValues"][0] = FriendlyName()


# Update the PKCS#12 maps

_pkcs12BagTypeMap = {
    id_keyBag: KeyBag(),
    id_pkcs8ShroudedKeyBag: PKCS8ShroudedKeyBag(),
    id_certBag: CertBag(),
    id_CRLBag: CRLBag(),
    id_secretBag: SecretBag(),
    id_safeContentsBag: SafeBag(),
}

pkcs12BagTypeMap.update(_pkcs12BagTypeMap)


_pkcs12CertBagMap = {
    _OID(certTypes, 1): univ.OctetString(),
    _OID(certTypes, 2): char.IA5String(),
}

pkcs12CertBagMap.update(_pkcs12CertBagMap)


_pkcs12CRLBagMap = {
    _OID(crlTypes, 1): univ.OctetString(),
}

pkcs12CRLBagMap.update(_pkcs12CRLBagMap)


# Update the Algorithm Identifier map

_algorithmIdentifierMapUpdate = {
    pbeWithSHAAnd128BitRC4: Pkcs_12PbeParams(),
    pbeWithSHAAnd40BitRC4: Pkcs_12PbeParams(),
    pbeWithSHAAnd3_KeyTripleDES_CBC: Pkcs_12PbeParams(),
    pbeWithSHAAnd2_KeyTripleDES_CBC: Pkcs_12PbeParams(),
    pbeWithSHAAnd128BitRC2_CBC: Pkcs_12PbeParams(),
    pbeWithSHAAnd40BitRC2_CBC: Pkcs_12PbeParams(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)


# Update the CMS Attribute map

_cmsAttributesMapUpdate = {
    pkcs_9_at_friendlyName: FriendlyName(),
    pkcs_9_at_localKeyId: univ.OctetString(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# SNMPv3 message syntax
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc3412.txt
#
from pyasn1.type import constraint, namedtype, univ

from pyasn1_modules import rfc1905


class ScopedPDU(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("contextEngineId", univ.OctetString()),
        namedtype.NamedType("contextName", univ.OctetString()),
        namedtype.NamedType("data", rfc1905.PDUs()),
    )


class ScopedPduData(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("plaintext", ScopedPDU()),
        namedtype.NamedType("encryptedPDU", univ.OctetString()),
    )


class HeaderData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "msgID",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, 2147483647)
            ),
        ),
        namedtype.NamedType(
            "msgMaxSize",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(484, 2147483647)
            ),
        ),
        namedtype.NamedType(
            "msgFlags",
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 1)
            ),
        ),
        namedtype.NamedType(
            "msgSecurityModel",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(1, 2147483647)
            ),
        ),
    )


class SNMPv3Message(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "msgVersion",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, 2147483647)
            ),
        ),
        namedtype.NamedType("msgGlobalData", HeaderData()),
        namedtype.NamedType("msgSecurityParameters", univ.OctetString()),
        namedtype.NamedType("msgData", ScopedPduData()),
    )



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Trust Anchor Format
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5914.txt

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc5280

MAX = float("inf")

Certificate = rfc5280.Certificate

Name = rfc5280.Name

Extensions = rfc5280.Extensions

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo

TBSCertificate = rfc5280.TBSCertificate

CertificatePolicies = rfc5280.CertificatePolicies

KeyIdentifier = rfc5280.KeyIdentifier

NameConstraints = rfc5280.NameConstraints


class CertPolicyFlags(univ.BitString):
    pass


CertPolicyFlags.namedValues = namedval.NamedValues(
    ("inhibitPolicyMapping", 0), ("requireExplicitPolicy", 1), ("inhibitAnyPolicy", 2)
)


class CertPathControls(univ.Sequence):
    pass


CertPathControls.componentType = namedtype.NamedTypes(
    namedtype.NamedType("taName", Name()),
    namedtype.OptionalNamedType(
        "certificate",
        Certificate().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "policySet",
        CertificatePolicies().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "policyFlags",
        CertPolicyFlags().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "nameConstr",
        NameConstraints().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.OptionalNamedType(
        "pathLenConstraint",
        univ.Integer()
        .subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX))
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)),
    ),
)


class TrustAnchorTitle(char.UTF8String):
    pass


TrustAnchorTitle.subtypeSpec = constraint.ValueSizeConstraint(1, 64)


class TrustAnchorInfoVersion(univ.Integer):
    pass


TrustAnchorInfoVersion.namedValues = namedval.NamedValues(("v1", 1))


class TrustAnchorInfo(univ.Sequence):
    pass


TrustAnchorInfo.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version", TrustAnchorInfoVersion().subtype(value="v1")
    ),
    namedtype.NamedType("pubKey", SubjectPublicKeyInfo()),
    namedtype.NamedType("keyId", KeyIdentifier()),
    namedtype.OptionalNamedType("taTitle", TrustAnchorTitle()),
    namedtype.OptionalNamedType("certPath", CertPathControls()),
    namedtype.OptionalNamedType(
        "exts",
        Extensions().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "taTitleLangTag",
        char.UTF8String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class TrustAnchorChoice(univ.Choice):
    pass


TrustAnchorChoice.componentType = namedtype.NamedTypes(
    namedtype.NamedType("certificate", Certificate()),
    namedtype.NamedType(
        "tbsCert",
        TBSCertificate().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "taInfo",
        TrustAnchorInfo().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
)


id_ct_trustAnchorList = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.34")


class TrustAnchorList(univ.SequenceOf):
    pass


TrustAnchorList.componentType = TrustAnchorChoice()
TrustAnchorList.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to add maps for use with opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# X.509 Extensions for IP Addresses and AS Identifiers
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3779.txt
#

from pyasn1.type import constraint, namedtype, tag, univ

from pyasn1_modules import rfc5280

# IP Address Delegation Extension

id_pe_ipAddrBlocks = univ.ObjectIdentifier("1.3.6.1.5.5.7.1.7")


class IPAddress(univ.BitString):
    pass


class IPAddressRange(univ.Sequence):
    pass


IPAddressRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("min", IPAddress()), namedtype.NamedType("max", IPAddress())
)


class IPAddressOrRange(univ.Choice):
    pass


IPAddressOrRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("addressPrefix", IPAddress()),
    namedtype.NamedType("addressRange", IPAddressRange()),
)


class IPAddressChoice(univ.Choice):
    pass


IPAddressChoice.componentType = namedtype.NamedTypes(
    namedtype.NamedType("inherit", univ.Null()),
    namedtype.NamedType(
        "addressesOrRanges", univ.SequenceOf(componentType=IPAddressOrRange())
    ),
)


class IPAddressFamily(univ.Sequence):
    pass


IPAddressFamily.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "addressFamily",
        univ.OctetString().subtype(subtypeSpec=constraint.ValueSizeConstraint(2, 3)),
    ),
    namedtype.NamedType("ipAddressChoice", IPAddressChoice()),
)


class IPAddrBlocks(univ.SequenceOf):
    pass


IPAddrBlocks.componentType = IPAddressFamily()


# Autonomous System Identifier Delegation Extension

id_pe_autonomousSysIds = univ.ObjectIdentifier("1.3.6.1.5.5.7.1.8")


class ASId(univ.Integer):
    pass


class ASRange(univ.Sequence):
    pass


ASRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("min", ASId()), namedtype.NamedType("max", ASId())
)


class ASIdOrRange(univ.Choice):
    pass


ASIdOrRange.componentType = namedtype.NamedTypes(
    namedtype.NamedType("id", ASId()), namedtype.NamedType("range", ASRange())
)


class ASIdentifierChoice(univ.Choice):
    pass


ASIdentifierChoice.componentType = namedtype.NamedTypes(
    namedtype.NamedType("inherit", univ.Null()),
    namedtype.NamedType("asIdsOrRanges", univ.SequenceOf(componentType=ASIdOrRange())),
)


class ASIdentifiers(univ.Sequence):
    pass


ASIdentifiers.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "asnum",
        ASIdentifierChoice().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "rdi",
        ASIdentifierChoice().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


# Map of Certificate Extension OIDs to Extensions is added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_pe_ipAddrBlocks: IPAddrBlocks(),
    id_pe_autonomousSysIds: ASIdentifiers(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to add maps for use with opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Certificate Extension for CMS Content Constraints (CCC)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6010.txt
#

from pyasn1.type import constraint, namedtype, namedval, univ

from pyasn1_modules import rfc5280

MAX = float("inf")


AttributeType = rfc5280.AttributeType

AttributeValue = rfc5280.AttributeValue


id_ct_anyContentType = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.0")


class AttrConstraint(univ.Sequence):
    pass


AttrConstraint.componentType = namedtype.NamedTypes(
    namedtype.NamedType("attrType", AttributeType()),
    namedtype.NamedType(
        "attrValues",
        univ.SetOf(componentType=AttributeValue()).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
)


class AttrConstraintList(univ.SequenceOf):
    pass


AttrConstraintList.componentType = AttrConstraint()
AttrConstraintList.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class ContentTypeGeneration(univ.Enumerated):
    pass


ContentTypeGeneration.namedValues = namedval.NamedValues(
    ("canSource", 0), ("cannotSource", 1)
)


class ContentTypeConstraint(univ.Sequence):
    pass


ContentTypeConstraint.componentType = namedtype.NamedTypes(
    namedtype.NamedType("contentType", univ.ObjectIdentifier()),
    namedtype.DefaultedNamedType(
        "canSource", ContentTypeGeneration().subtype(value="canSource")
    ),
    namedtype.OptionalNamedType("attrConstraints", AttrConstraintList()),
)


# CMS Content Constraints (CCC) Extension and Object Identifier

id_pe_cmsContentConstraints = univ.ObjectIdentifier("1.3.6.1.5.5.7.1.18")


class CMSContentConstraints(univ.SequenceOf):
    pass


CMSContentConstraints.componentType = ContentTypeConstraint()
CMSContentConstraints.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


# Map of Certificate Extension OIDs to Extensions
# To be added to the ones that are in rfc5280.py

_certificateExtensionsMap = {
    id_pe_cmsContentConstraints: CMSContentConstraints(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMap)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Diffie-Hellman Key Agreement
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc2631.txt
# https://www.rfc-editor.org/errata/eid5897
#

from pyasn1.type import constraint, namedtype, tag, univ


class KeySpecificInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algorithm", univ.ObjectIdentifier()),
        namedtype.NamedType(
            "counter",
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(4, 4)
            ),
        ),
    )


class OtherInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("keyInfo", KeySpecificInfo()),
        namedtype.OptionalNamedType(
            "partyAInfo",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "suppPubInfo",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der import decoder as der_decoder
from pyasn1.codec.der import encoder as der_encoder
from pyasn1.type import univ

from pyasn1_modules import pem, rfc4055, rfc5280


class PSSDefautTestCase(unittest.TestCase):
    pss_default_pem_text = "MAsGCSqGSIb3DQEBCg=="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pss_default_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertTrue(rfc4055.id_RSASSA_PSS, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pss_default_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertFalse(asn1Object["parameters"].hasValue())


class PSSSHA512TestCase(unittest.TestCase):
    pss_sha512_pem_text = (
        "MDwGCSqGSIb3DQEBCjAvoA8wDQYJYIZIAWUDBAIDBQChHDAaBg"
        "kqhkiG9w0BAQgwDQYJYIZIAWUDBAIDBQA="
    )

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pss_sha512_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertTrue(rfc4055.id_RSASSA_PSS, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pss_sha512_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertTrue(asn1Object["parameters"].hasValue())
        self.assertTrue(20, asn1Object["parameters"]["saltLength"])


class OAEPDefautTestCase(unittest.TestCase):
    oaep_default_pem_text = "MAsGCSqGSIb3DQEBBw=="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.oaep_default_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertTrue(rfc4055.id_RSAES_OAEP, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.oaep_default_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertFalse(asn1Object["parameters"].hasValue())


class OAEPSHA256TestCase(unittest.TestCase):
    oaep_sha256_pem_text = (
        "MDwGCSqGSIb3DQEBBzAvoA8wDQYJYIZIAWUDBAIBBQChHDAaB"
        "gkqhkiG9w0BAQgwDQYJYIZIAWUDBAIBBQA="
    )

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.oaep_sha256_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertTrue(rfc4055.id_RSAES_OAEP, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.oaep_sha256_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertTrue(asn1Object["parameters"].hasValue())

        oaep_p = asn1Object["parameters"]

        self.assertEqual(univ.Null(""), oaep_p["hashFunc"]["parameters"])
        self.assertEqual(
            univ.Null(""), oaep_p["maskGenFunc"]["parameters"]["parameters"]
        )


class OAEPFullTestCase(unittest.TestCase):
    oaep_full_pem_text = (
        "MFMGCSqGSIb3DQEBBzBGoA8wDQYJYIZIAWUDBAICBQChHDAaBgk"
        "qhkiG9w0BAQgwDQYJYIZIAWUDBAICBQCiFTATBgkqhkiG9w0BAQ"
        "kEBmZvb2Jhcg=="
    )

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.oaep_full_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())

        self.assertTrue(rfc4055.id_RSAES_OAEP, asn1Object[0])

        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.oaep_full_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True
        )
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

        self.assertTrue(asn1Object["parameters"].hasValue())

        oaep_p = asn1Object["parameters"]

        self.assertEqual(univ.Null(""), oaep_p["hashFunc"]["parameters"])
        self.assertEqual(
            univ.Null(""), oaep_p["maskGenFunc"]["parameters"]["parameters"]
        )
        self.assertEqual(
            univ.OctetString(value="foobar"), oaep_p["pSourceFunc"]["parameters"]
        )


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# PKCS#7 message syntax
#
# ASN.1 source from:
# https://opensource.apple.com/source/Security/Security-55179.1/libsecurity_asn1/asn1/pkcs7.asn.auto.html
#
# Sample captures from:
# openssl crl2pkcs7 -nocrl -certfile cert1.cer -out outfile.p7b
#
from pyasn1_modules.rfc2459 import *


class Attribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeType()),
        namedtype.NamedType("values", univ.SetOf(componentType=AttributeValue())),
    )


class AttributeValueAssertion(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("attributeType", AttributeType()),
        namedtype.NamedType(
            "attributeValue",
            AttributeValue(),
            openType=opentype.OpenType("type", certificateAttributesMap),
        ),
    )


pkcs_7 = univ.ObjectIdentifier("1.2.840.113549.1.7")
data = univ.ObjectIdentifier("1.2.840.113549.1.7.1")
signedData = univ.ObjectIdentifier("1.2.840.113549.1.7.2")
envelopedData = univ.ObjectIdentifier("1.2.840.113549.1.7.3")
signedAndEnvelopedData = univ.ObjectIdentifier("1.2.840.113549.1.7.4")
digestedData = univ.ObjectIdentifier("1.2.840.113549.1.7.5")
encryptedData = univ.ObjectIdentifier("1.2.840.113549.1.7.6")


class ContentType(univ.ObjectIdentifier):
    pass


class ContentEncryptionAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class EncryptedContent(univ.OctetString):
    pass


contentTypeMap = {}


class EncryptedContentInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("contentType", ContentType()),
        namedtype.NamedType(
            "contentEncryptionAlgorithm", ContentEncryptionAlgorithmIdentifier()
        ),
        namedtype.OptionalNamedType(
            "encryptedContent",
            EncryptedContent().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
            openType=opentype.OpenType("contentType", contentTypeMap),
        ),
    )


class Version(univ.Integer):  # overrides x509.Version
    pass


class EncryptedData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("encryptedContentInfo", EncryptedContentInfo()),
    )


class DigestAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class DigestAlgorithmIdentifiers(univ.SetOf):
    componentType = DigestAlgorithmIdentifier()


class Digest(univ.OctetString):
    pass


class ContentInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("contentType", ContentType()),
        namedtype.OptionalNamedType(
            "content",
            univ.Any().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
            openType=opentype.OpenType("contentType", contentTypeMap),
        ),
    )


class DigestedData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("digestAlgorithm", DigestAlgorithmIdentifier()),
        namedtype.NamedType("contentInfo", ContentInfo()),
        namedtype.NamedType("digest", Digest()),
    )


class IssuerAndSerialNumber(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("issuer", Name()),
        namedtype.NamedType("serialNumber", CertificateSerialNumber()),
    )


class KeyEncryptionAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class EncryptedKey(univ.OctetString):
    pass


class RecipientInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
        namedtype.NamedType(
            "keyEncryptionAlgorithm", KeyEncryptionAlgorithmIdentifier()
        ),
        namedtype.NamedType("encryptedKey", EncryptedKey()),
    )


class RecipientInfos(univ.SetOf):
    componentType = RecipientInfo()


class Attributes(univ.SetOf):
    componentType = Attribute()


class ExtendedCertificateInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("certificate", Certificate()),
        namedtype.NamedType("attributes", Attributes()),
    )


class SignatureAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class Signature(univ.BitString):
    pass


class ExtendedCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("extendedCertificateInfo", ExtendedCertificateInfo()),
        namedtype.NamedType("signatureAlgorithm", SignatureAlgorithmIdentifier()),
        namedtype.NamedType("signature", Signature()),
    )


class ExtendedCertificateOrCertificate(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certificate", Certificate()),
        namedtype.NamedType(
            "extendedCertificate",
            ExtendedCertificate().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


class ExtendedCertificatesAndCertificates(univ.SetOf):
    componentType = ExtendedCertificateOrCertificate()


class SerialNumber(univ.Integer):
    pass


class CRLEntry(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("userCertificate", SerialNumber()),
        namedtype.NamedType("revocationDate", useful.UTCTime()),
    )


class TBSCertificateRevocationList(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signature", AlgorithmIdentifier()),
        namedtype.NamedType("issuer", Name()),
        namedtype.NamedType("lastUpdate", useful.UTCTime()),
        namedtype.NamedType("nextUpdate", useful.UTCTime()),
        namedtype.OptionalNamedType(
            "revokedCertificates", univ.SequenceOf(componentType=CRLEntry())
        ),
    )


class CertificateRevocationList(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "tbsCertificateRevocationList", TBSCertificateRevocationList()
        ),
        namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
    )


class CertificateRevocationLists(univ.SetOf):
    componentType = CertificateRevocationList()


class DigestEncryptionAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class EncryptedDigest(univ.OctetString):
    pass


class SignerInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("issuerAndSerialNumber", IssuerAndSerialNumber()),
        namedtype.NamedType("digestAlgorithm", DigestAlgorithmIdentifier()),
        namedtype.OptionalNamedType(
            "authenticatedAttributes",
            Attributes().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "digestEncryptionAlgorithm", DigestEncryptionAlgorithmIdentifier()
        ),
        namedtype.NamedType("encryptedDigest", EncryptedDigest()),
        namedtype.OptionalNamedType(
            "unauthenticatedAttributes",
            Attributes().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
    )


class SignerInfos(univ.SetOf):
    componentType = SignerInfo()


class SignedAndEnvelopedData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("recipientInfos", RecipientInfos()),
        namedtype.NamedType("digestAlgorithms", DigestAlgorithmIdentifiers()),
        namedtype.NamedType("encryptedContentInfo", EncryptedContentInfo()),
        namedtype.OptionalNamedType(
            "certificates",
            ExtendedCertificatesAndCertificates().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "crls",
            CertificateRevocationLists().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.NamedType("signerInfos", SignerInfos()),
    )


class EnvelopedData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("recipientInfos", RecipientInfos()),
        namedtype.NamedType("encryptedContentInfo", EncryptedContentInfo()),
    )


class DigestInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("digestAlgorithm", DigestAlgorithmIdentifier()),
        namedtype.NamedType("digest", Digest()),
    )


class SignedData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.OptionalNamedType("digestAlgorithms", DigestAlgorithmIdentifiers()),
        namedtype.NamedType("contentInfo", ContentInfo()),
        namedtype.OptionalNamedType(
            "certificates",
            ExtendedCertificatesAndCertificates().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "crls",
            CertificateRevocationLists().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType("signerInfos", SignerInfos()),
    )


class Data(univ.OctetString):
    pass


_contentTypeMapUpdate = {
    data: Data(),
    signedData: SignedData(),
    envelopedData: EnvelopedData(),
    signedAndEnvelopedData: SignedAndEnvelopedData(),
    digestedData: DigestedData(),
    encryptedData: EncryptedData(),
}

contentTypeMap.update(_contentTypeMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Alternative Challenge Password Attributes for EST
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7894.txt
#

from pyasn1.type import char, constraint, namedtype, univ

from pyasn1_modules import rfc5652, rfc6402, rfc7191

# SingleAttribute is the same as Attribute in RFC 5652, except that the
# attrValues SET must have one and only one member

Attribute = rfc7191.SingleAttribute


# DirectoryString is the same as RFC 5280, except the length is limited to 255


class DirectoryString(univ.Choice):
    pass


DirectoryString.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "teletexString",
        char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 255)
        ),
    ),
    namedtype.NamedType(
        "printableString",
        char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 255)
        ),
    ),
    namedtype.NamedType(
        "universalString",
        char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, 255)
        ),
    ),
    namedtype.NamedType(
        "utf8String",
        char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 255)),
    ),
    namedtype.NamedType(
        "bmpString",
        char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 255)),
    ),
)


# OTP Challenge Attribute

id_aa_otpChallenge = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.56")

ub_aa_otpChallenge = univ.Integer(255)

otpChallenge = Attribute()
otpChallenge["attrType"] = id_aa_otpChallenge
otpChallenge["attrValues"][0] = DirectoryString()


# Revocation Challenge Attribute

id_aa_revocationChallenge = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.57")

ub_aa_revocationChallenge = univ.Integer(255)

revocationChallenge = Attribute()
revocationChallenge["attrType"] = id_aa_revocationChallenge
revocationChallenge["attrValues"][0] = DirectoryString()


#  EST Identity Linking Attribute

id_aa_estIdentityLinking = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.58")

ub_aa_est_identity_linking = univ.Integer(255)

estIdentityLinking = Attribute()
estIdentityLinking["attrType"] = id_aa_estIdentityLinking
estIdentityLinking["attrValues"][0] = DirectoryString()


# Map of Attribute Type OIDs to Attributes added to the
# ones that are in rfc6402.py

_cmcControlAttributesMapUpdate = {
    id_aa_otpChallenge: DirectoryString(),
    id_aa_revocationChallenge: DirectoryString(),
    id_aa_estIdentityLinking: DirectoryString(),
}

rfc6402.cmcControlAttributesMap.update(_cmcControlAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Stanisław Pitucha with asn1ate tool.
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# An Internet Attribute Certificate Profile for Authorization
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc3281.txt
#
from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc3280

MAX = float("inf")


def _buildOid(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


class ObjectDigestInfo(univ.Sequence):
    pass


ObjectDigestInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "digestedObjectType",
        univ.Enumerated(
            namedValues=namedval.NamedValues(
                ("publicKey", 0), ("publicKeyCert", 1), ("otherObjectTypes", 2)
            )
        ),
    ),
    namedtype.OptionalNamedType("otherObjectTypeID", univ.ObjectIdentifier()),
    namedtype.NamedType("digestAlgorithm", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("objectDigest", univ.BitString()),
)


class IssuerSerial(univ.Sequence):
    pass


IssuerSerial.componentType = namedtype.NamedTypes(
    namedtype.NamedType("issuer", rfc3280.GeneralNames()),
    namedtype.NamedType("serial", rfc3280.CertificateSerialNumber()),
    namedtype.OptionalNamedType("issuerUID", rfc3280.UniqueIdentifier()),
)


class TargetCert(univ.Sequence):
    pass


TargetCert.componentType = namedtype.NamedTypes(
    namedtype.NamedType("targetCertificate", IssuerSerial()),
    namedtype.OptionalNamedType("targetName", rfc3280.GeneralName()),
    namedtype.OptionalNamedType("certDigestInfo", ObjectDigestInfo()),
)


class Target(univ.Choice):
    pass


Target.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "targetName",
        rfc3280.GeneralName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "targetGroup",
        rfc3280.GeneralName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "targetCert",
        TargetCert().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
)


class Targets(univ.SequenceOf):
    pass


Targets.componentType = Target()


class ProxyInfo(univ.SequenceOf):
    pass


ProxyInfo.componentType = Targets()

id_at_role = _buildOid(rfc3280.id_at, 72)

id_pe_aaControls = _buildOid(rfc3280.id_pe, 6)

id_ce_targetInformation = _buildOid(rfc3280.id_ce, 55)

id_pe_ac_auditIdentity = _buildOid(rfc3280.id_pe, 4)


class ClassList(univ.BitString):
    pass


ClassList.namedValues = namedval.NamedValues(
    ("unmarked", 0),
    ("unclassified", 1),
    ("restricted", 2),
    ("confidential", 3),
    ("secret", 4),
    ("topSecret", 5),
)


class SecurityCategory(univ.Sequence):
    pass


SecurityCategory.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "type",
        univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "value",
        univ.Any().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class Clearance(univ.Sequence):
    pass


Clearance.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "policyId",
        univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.DefaultedNamedType(
        "classList",
        ClassList()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
        .subtype(value="unclassified"),
    ),
    namedtype.OptionalNamedType(
        "securityCategories",
        univ.SetOf(componentType=SecurityCategory()).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class AttCertVersion(univ.Integer):
    pass


AttCertVersion.namedValues = namedval.NamedValues(("v2", 1))

id_aca = _buildOid(rfc3280.id_pkix, 10)

id_at_clearance = _buildOid(2, 5, 1, 5, 55)


class AttrSpec(univ.SequenceOf):
    pass


AttrSpec.componentType = univ.ObjectIdentifier()


class AAControls(univ.Sequence):
    pass


AAControls.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "pathLenConstraint",
        univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
    ),
    namedtype.OptionalNamedType(
        "permittedAttrs",
        AttrSpec().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "excludedAttrs",
        AttrSpec().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.DefaultedNamedType("permitUnSpecified", univ.Boolean().subtype(value=1)),
)


class AttCertValidityPeriod(univ.Sequence):
    pass


AttCertValidityPeriod.componentType = namedtype.NamedTypes(
    namedtype.NamedType("notBeforeTime", useful.GeneralizedTime()),
    namedtype.NamedType("notAfterTime", useful.GeneralizedTime()),
)


id_aca_authenticationInfo = _buildOid(id_aca, 1)


class V2Form(univ.Sequence):
    pass


V2Form.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("issuerName", rfc3280.GeneralNames()),
    namedtype.OptionalNamedType(
        "baseCertificateID",
        IssuerSerial().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "objectDigestInfo",
        ObjectDigestInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class AttCertIssuer(univ.Choice):
    pass


AttCertIssuer.componentType = namedtype.NamedTypes(
    namedtype.NamedType("v1Form", rfc3280.GeneralNames()),
    namedtype.NamedType(
        "v2Form",
        V2Form().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
)


class Holder(univ.Sequence):
    pass


Holder.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "baseCertificateID",
        IssuerSerial().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "entityName",
        rfc3280.GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "objectDigestInfo",
        ObjectDigestInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
        ),
    ),
)


class AttributeCertificateInfo(univ.Sequence):
    pass


AttributeCertificateInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("version", AttCertVersion()),
    namedtype.NamedType("holder", Holder()),
    namedtype.NamedType("issuer", AttCertIssuer()),
    namedtype.NamedType("signature", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("serialNumber", rfc3280.CertificateSerialNumber()),
    namedtype.NamedType("attrCertValidityPeriod", AttCertValidityPeriod()),
    namedtype.NamedType(
        "attributes", univ.SequenceOf(componentType=rfc3280.Attribute())
    ),
    namedtype.OptionalNamedType("issuerUniqueID", rfc3280.UniqueIdentifier()),
    namedtype.OptionalNamedType("extensions", rfc3280.Extensions()),
)


class AttributeCertificate(univ.Sequence):
    pass


AttributeCertificate.componentType = namedtype.NamedTypes(
    namedtype.NamedType("acinfo", AttributeCertificateInfo()),
    namedtype.NamedType("signatureAlgorithm", rfc3280.AlgorithmIdentifier()),
    namedtype.NamedType("signatureValue", univ.BitString()),
)

id_mod = _buildOid(rfc3280.id_pkix, 0)

id_mod_attribute_cert = _buildOid(id_mod, 12)

id_aca_accessIdentity = _buildOid(id_aca, 2)


class RoleSyntax(univ.Sequence):
    pass


RoleSyntax.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "roleAuthority",
        rfc3280.GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "roleName",
        rfc3280.GeneralName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)

id_aca_chargingIdentity = _buildOid(id_aca, 3)


class ACClearAttrs(univ.Sequence):
    pass


ACClearAttrs.componentType = namedtype.NamedTypes(
    namedtype.NamedType("acIssuer", rfc3280.GeneralName()),
    namedtype.NamedType("acSerial", univ.Integer()),
    namedtype.NamedType("attrs", univ.SequenceOf(componentType=rfc3280.Attribute())),
)

id_aca_group = _buildOid(id_aca, 4)

id_pe_ac_proxying = _buildOid(rfc3280.id_pe, 10)


class SvceAuthInfo(univ.Sequence):
    pass


SvceAuthInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("service", rfc3280.GeneralName()),
    namedtype.NamedType("ident", rfc3280.GeneralName()),
    namedtype.OptionalNamedType("authInfo", univ.OctetString()),
)


class IetfAttrSyntax(univ.Sequence):
    pass


IetfAttrSyntax.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "policyAuthority",
        rfc3280.GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "values",
        univ.SequenceOf(
            componentType=univ.Choice(
                componentType=namedtype.NamedTypes(
                    namedtype.NamedType("octets", univ.OctetString()),
                    namedtype.NamedType("oid", univ.ObjectIdentifier()),
                    namedtype.NamedType("string", char.UTF8String()),
                )
            )
        ),
    ),
)

id_aca_encAttrs = _buildOid(id_aca, 6)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CMS Algorithm Identifier Protection Attribute
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6211.txt
#

from pyasn1.type import constraint, namedtype, tag, univ

from pyasn1_modules import rfc5652

# Imports from RFC 5652

DigestAlgorithmIdentifier = rfc5652.DigestAlgorithmIdentifier

MessageAuthenticationCodeAlgorithm = rfc5652.MessageAuthenticationCodeAlgorithm

SignatureAlgorithmIdentifier = rfc5652.SignatureAlgorithmIdentifier


# CMS Algorithm Protection attribute

id_aa_cmsAlgorithmProtect = univ.ObjectIdentifier("1.2.840.113549.1.9.52")


class CMSAlgorithmProtection(univ.Sequence):
    pass


CMSAlgorithmProtection.componentType = namedtype.NamedTypes(
    namedtype.NamedType("digestAlgorithm", DigestAlgorithmIdentifier()),
    namedtype.OptionalNamedType(
        "signatureAlgorithm",
        SignatureAlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "macAlgorithm",
        MessageAuthenticationCodeAlgorithm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)

CMSAlgorithmProtection.subtypeSpec = constraint.ConstraintsUnion(
    constraint.WithComponentsConstraint(
        ("signatureAlgorithm", constraint.ComponentPresentConstraint()),
        ("macAlgorithm", constraint.ComponentAbsentConstraint()),
    ),
    constraint.WithComponentsConstraint(
        ("signatureAlgorithm", constraint.ComponentAbsentConstraint()),
        ("macAlgorithm", constraint.ComponentPresentConstraint()),
    ),
)


aa_cmsAlgorithmProtection = rfc5652.Attribute()
aa_cmsAlgorithmProtection["attrType"] = id_aa_cmsAlgorithmProtect
aa_cmsAlgorithmProtection["attrValues"][0] = CMSAlgorithmProtection()


# Map of Attribute Type OIDs to Attributes are
# added to the ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_cmsAlgorithmProtect: CMSAlgorithmProtection(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# X.509 certificate Request Message Format (CRMF) syntax
#
# ASN.1 source from:
# http://tools.ietf.org/html/rfc2511
#
# Sample captures could be obtained with OpenSSL
#
from pyasn1_modules import rfc2315
from pyasn1_modules.rfc2459 import *

MAX = float("inf")

id_pkix = univ.ObjectIdentifier("1.3.6.1.5.5.7")
id_pkip = univ.ObjectIdentifier("1.3.6.1.5.5.7.5")
id_regCtrl = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1")
id_regCtrl_regToken = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1.1")
id_regCtrl_authenticator = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1.2")
id_regCtrl_pkiPublicationInfo = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1.3")
id_regCtrl_pkiArchiveOptions = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1.4")
id_regCtrl_oldCertID = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1.5")
id_regCtrl_protocolEncrKey = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.1.6")
id_regInfo = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.2")
id_regInfo_utf8Pairs = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.2.1")
id_regInfo_certReq = univ.ObjectIdentifier("1.3.6.1.5.5.7.5.2.2")


# This should be in PKIX Certificate Extensions module


class GeneralName(univ.OctetString):
    pass


# end of PKIX Certificate Extensions module


class UTF8Pairs(char.UTF8String):
    pass


class ProtocolEncrKey(SubjectPublicKeyInfo):
    pass


class CertId(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("issuer", GeneralName()),
        namedtype.NamedType("serialNumber", univ.Integer()),
    )


class OldCertId(CertId):
    pass


class KeyGenParameters(univ.OctetString):
    pass


class EncryptedValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "intendedAlg",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "symmAlg",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "encSymmKey",
            univ.BitString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "keyAlg",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "valueHint",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.NamedType("encValue", univ.BitString()),
    )


class EncryptedKey(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("encryptedValue", EncryptedValue()),
        namedtype.NamedType(
            "envelopedData",
            rfc2315.EnvelopedData().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


class PKIArchiveOptions(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "encryptedPrivKey",
            EncryptedKey().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "keyGenParameters",
            KeyGenParameters().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "archiveRemGenPrivKey",
            univ.Boolean().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class SinglePubInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "pubMethod",
            univ.Integer(
                namedValues=namedval.NamedValues(
                    ("dontCare", 0), ("x500", 1), ("web", 2), ("ldap", 3)
                )
            ),
        ),
        namedtype.OptionalNamedType("pubLocation", GeneralName()),
    )


class PKIPublicationInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "action",
            univ.Integer(
                namedValues=namedval.NamedValues(
                    ("dontPublish", 0), ("pleasePublish", 1)
                )
            ),
        ),
        namedtype.OptionalNamedType(
            "pubInfos",
            univ.SequenceOf(componentType=SinglePubInfo()).subtype(
                sizeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


class Authenticator(char.UTF8String):
    pass


class RegToken(char.UTF8String):
    pass


class SubsequentMessage(univ.Integer):
    namedValues = namedval.NamedValues(("encrCert", 0), ("challengeResp", 1))


class POPOPrivKey(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "thisMessage",
            univ.BitString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "subsequentMessage",
            SubsequentMessage().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "dhMAC",
            univ.BitString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class PBMParameter(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("salt", univ.OctetString()),
        namedtype.NamedType("owf", AlgorithmIdentifier()),
        namedtype.NamedType("iterationCount", univ.Integer()),
        namedtype.NamedType("mac", AlgorithmIdentifier()),
    )


class PKMACValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algId", AlgorithmIdentifier()),
        namedtype.NamedType("value", univ.BitString()),
    )


class POPOSigningKeyInput(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "authInfo",
            univ.Choice(
                componentType=namedtype.NamedTypes(
                    namedtype.NamedType(
                        "sender",
                        GeneralName().subtype(
                            implicitTag=tag.Tag(
                                tag.tagClassContext, tag.tagFormatSimple, 0
                            )
                        ),
                    ),
                    namedtype.NamedType("publicKeyMAC", PKMACValue()),
                )
            ),
        ),
        namedtype.NamedType("publicKey", SubjectPublicKeyInfo()),
    )


class POPOSigningKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "poposkInput",
            POPOSigningKeyInput().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType("algorithmIdentifier", AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
    )


class ProofOfPossession(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "raVerified",
            univ.Null().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "signature",
            POPOSigningKey().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.NamedType(
            "keyEncipherment",
            POPOPrivKey().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.NamedType(
            "keyAgreement",
            POPOPrivKey().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
    )


class Controls(univ.SequenceOf):
    componentType = AttributeTypeAndValue()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class OptionalValidity(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "notBefore",
            Time().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "notAfter",
            Time().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class CertTemplate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "version",
            Version().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "serialNumber",
            univ.Integer().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "signingAlg",
            AlgorithmIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "issuer",
            Name().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "validity",
            OptionalValidity().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 4)
            ),
        ),
        namedtype.OptionalNamedType(
            "subject",
            Name().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 5)
            ),
        ),
        namedtype.OptionalNamedType(
            "publicKey",
            SubjectPublicKeyInfo().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 6)
            ),
        ),
        namedtype.OptionalNamedType(
            "issuerUID",
            UniqueIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
            ),
        ),
        namedtype.OptionalNamedType(
            "subjectUID",
            UniqueIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 8)
            ),
        ),
        namedtype.OptionalNamedType(
            "extensions",
            Extensions().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 9)
            ),
        ),
    )


class CertRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certReqId", univ.Integer()),
        namedtype.NamedType("certTemplate", CertTemplate()),
        namedtype.OptionalNamedType("controls", Controls()),
    )


class CertReq(CertRequest):
    pass


class CertReqMsg(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certReq", CertRequest()),
        namedtype.OptionalNamedType("pop", ProofOfPossession()),
        namedtype.OptionalNamedType(
            "regInfo",
            univ.SequenceOf(componentType=AttributeTypeAndValue()).subtype(
                sizeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


class CertReqMessages(univ.SequenceOf):
    componentType = CertReqMsg()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# RPKI Route Origin Authorizations (ROAs)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6482.txt
# https://www.rfc-editor.org/errata/eid5881
#

from pyasn1.type import constraint, namedtype, tag, univ

from pyasn1_modules import rfc5652

MAX = float("inf")


id_ct_routeOriginAuthz = univ.ObjectIdentifier("1.2.840.113549.1.9.16.1.24")


class ASID(univ.Integer):
    pass


class IPAddress(univ.BitString):
    pass


class ROAIPAddress(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("address", IPAddress()),
        namedtype.OptionalNamedType("maxLength", univ.Integer()),
    )


class ROAIPAddressFamily(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "addressFamily",
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(2, 3)
            ),
        ),
        namedtype.NamedType(
            "addresses",
            univ.SequenceOf(componentType=ROAIPAddress()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


class RouteOriginAttestation(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            univ.Integer()
            .subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
            .subtype(value=0),
        ),
        namedtype.NamedType("asID", ASID()),
        namedtype.NamedType(
            "ipAddrBlocks",
            univ.SequenceOf(componentType=ROAIPAddressFamily()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


# Map of Content Type OIDs to Content Types added to the
# ones that are in rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_routeOriginAuthz: RouteOriginAttestation(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Securing Header Fields with S/MIME
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7508.txt
# https://www.rfc-editor.org/errata/eid5875
#

import string

from pyasn1.type import char, constraint, namedtype, namedval, univ

from pyasn1_modules import rfc5652

MAX = float("inf")


class Algorithm(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ("canonAlgorithmSimple", 0), ("canonAlgorithmRelaxed", 1)
    )


class HeaderFieldStatus(univ.Integer):
    namedValues = namedval.NamedValues(
        ("duplicated", 0), ("deleted", 1), ("modified", 2)
    )


class HeaderFieldName(char.VisibleString):
    subtypeSpec = constraint.PermittedAlphabetConstraint(
        *string.printable
    ) - constraint.PermittedAlphabetConstraint(":")


class HeaderFieldValue(char.UTF8String):
    pass


class HeaderField(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("field-Name", HeaderFieldName()),
        namedtype.NamedType("field-Value", HeaderFieldValue()),
        namedtype.DefaultedNamedType(
            "field-Status", HeaderFieldStatus().subtype(value="duplicated")
        ),
    )


class HeaderFields(univ.SequenceOf):
    componentType = HeaderField()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class SecureHeaderFields(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("canonAlgorithm", Algorithm()),
        namedtype.NamedType("secHeaderFields", HeaderFields()),
    )


id_aa = univ.ObjectIdentifier(
    (
        1,
        2,
        840,
        113549,
        1,
        9,
        16,
        2,
    )
)

id_aa_secureHeaderFieldsIdentifier = id_aa + (55,)


# Map of Attribute Type OIDs to Attributes added to the
# ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_secureHeaderFieldsIdentifier: SecureHeaderFields(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Trust Anchor Format
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5934.txt

from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc2985, rfc5280, rfc5652, rfc5914

MAX = float("inf")


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))
    return univ.ObjectIdentifier(output)


# Imports from RFC 2985

SingleAttribute = rfc2985.SingleAttribute


# Imports from RFC5914

CertPathControls = rfc5914.CertPathControls

TrustAnchorChoice = rfc5914.TrustAnchorChoice

TrustAnchorTitle = rfc5914.TrustAnchorTitle


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

AnotherName = rfc5280.AnotherName

Attribute = rfc5280.Attribute

Certificate = rfc5280.Certificate

CertificateSerialNumber = rfc5280.CertificateSerialNumber

Extension = rfc5280.Extension

Extensions = rfc5280.Extensions

KeyIdentifier = rfc5280.KeyIdentifier

Name = rfc5280.Name

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo

TBSCertificate = rfc5280.TBSCertificate

Validity = rfc5280.Validity


# Object Identifier Arc for TAMP Message Content Types

id_tamp = univ.ObjectIdentifier("2.16.840.1.101.2.1.2.77")


# TAMP Status Query Message

id_ct_TAMP_statusQuery = _OID(id_tamp, 1)


class TAMPVersion(univ.Integer):
    pass


TAMPVersion.namedValues = namedval.NamedValues(("v1", 1), ("v2", 2))


class TerseOrVerbose(univ.Enumerated):
    pass


TerseOrVerbose.namedValues = namedval.NamedValues(("terse", 1), ("verbose", 2))


class HardwareSerialEntry(univ.Choice):
    pass


HardwareSerialEntry.componentType = namedtype.NamedTypes(
    namedtype.NamedType("all", univ.Null()),
    namedtype.NamedType("single", univ.OctetString()),
    namedtype.NamedType(
        "block",
        univ.Sequence(
            componentType=namedtype.NamedTypes(
                namedtype.NamedType("low", univ.OctetString()),
                namedtype.NamedType("high", univ.OctetString()),
            )
        ),
    ),
)


class HardwareModules(univ.Sequence):
    pass


HardwareModules.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hwType", univ.ObjectIdentifier()),
    namedtype.NamedType(
        "hwSerialEntries",
        univ.SequenceOf(componentType=HardwareSerialEntry()).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
)


class HardwareModuleIdentifierList(univ.SequenceOf):
    pass


HardwareModuleIdentifierList.componentType = HardwareModules()
HardwareModuleIdentifierList.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class Community(univ.ObjectIdentifier):
    pass


class CommunityIdentifierList(univ.SequenceOf):
    pass


CommunityIdentifierList.componentType = Community()
CommunityIdentifierList.subtypeSpec = constraint.ValueSizeConstraint(0, MAX)


class TargetIdentifier(univ.Choice):
    pass


TargetIdentifier.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "hwModules",
        HardwareModuleIdentifierList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "communities",
        CommunityIdentifierList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType(
        "allModules",
        univ.Null().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.NamedType(
        "uri",
        char.IA5String().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
    namedtype.NamedType(
        "otherName",
        AnotherName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
        ),
    ),
)


class SeqNumber(univ.Integer):
    pass


SeqNumber.subtypeSpec = constraint.ValueRangeConstraint(0, 9223372036854775807)


class TAMPMsgRef(univ.Sequence):
    pass


TAMPMsgRef.componentType = namedtype.NamedTypes(
    namedtype.NamedType("target", TargetIdentifier()),
    namedtype.NamedType("seqNum", SeqNumber()),
)


class TAMPStatusQuery(univ.Sequence):
    pass


TAMPStatusQuery.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.DefaultedNamedType(
        "terse",
        TerseOrVerbose()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
        .subtype(value="verbose"),
    ),
    namedtype.NamedType("query", TAMPMsgRef()),
)


tamp_status_query = rfc5652.ContentInfo()
tamp_status_query["contentType"] = id_ct_TAMP_statusQuery
tamp_status_query["content"] = TAMPStatusQuery()


# TAMP Status Response Message

id_ct_TAMP_statusResponse = _OID(id_tamp, 2)


class KeyIdentifiers(univ.SequenceOf):
    pass


KeyIdentifiers.componentType = KeyIdentifier()
KeyIdentifiers.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class TrustAnchorChoiceList(univ.SequenceOf):
    pass


TrustAnchorChoiceList.componentType = TrustAnchorChoice()
TrustAnchorChoiceList.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class TAMPSequenceNumber(univ.Sequence):
    pass


TAMPSequenceNumber.componentType = namedtype.NamedTypes(
    namedtype.NamedType("keyId", KeyIdentifier()),
    namedtype.NamedType("seqNumber", SeqNumber()),
)


class TAMPSequenceNumbers(univ.SequenceOf):
    pass


TAMPSequenceNumbers.componentType = TAMPSequenceNumber()
TAMPSequenceNumbers.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class TerseStatusResponse(univ.Sequence):
    pass


TerseStatusResponse.componentType = namedtype.NamedTypes(
    namedtype.NamedType("taKeyIds", KeyIdentifiers()),
    namedtype.OptionalNamedType("communities", CommunityIdentifierList()),
)


class VerboseStatusResponse(univ.Sequence):
    pass


VerboseStatusResponse.componentType = namedtype.NamedTypes(
    namedtype.NamedType("taInfo", TrustAnchorChoiceList()),
    namedtype.OptionalNamedType(
        "continPubKeyDecryptAlg",
        AlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "communities",
        CommunityIdentifierList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "tampSeqNumbers",
        TAMPSequenceNumbers().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class StatusResponse(univ.Choice):
    pass


StatusResponse.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "terseResponse",
        TerseStatusResponse().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "verboseResponse",
        VerboseStatusResponse().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class TAMPStatusResponse(univ.Sequence):
    pass


TAMPStatusResponse.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("query", TAMPMsgRef()),
    namedtype.NamedType("response", StatusResponse()),
    namedtype.DefaultedNamedType("usesApex", univ.Boolean().subtype(value=1)),
)


tamp_status_response = rfc5652.ContentInfo()
tamp_status_response["contentType"] = id_ct_TAMP_statusResponse
tamp_status_response["content"] = TAMPStatusResponse()


# Trust Anchor Update Message

id_ct_TAMP_update = _OID(id_tamp, 3)


class TBSCertificateChangeInfo(univ.Sequence):
    pass


TBSCertificateChangeInfo.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType("serialNumber", CertificateSerialNumber()),
    namedtype.OptionalNamedType(
        "signature",
        AlgorithmIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "issuer",
        Name().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "validity",
        Validity().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.OptionalNamedType(
        "subject",
        Name().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
        ),
    ),
    namedtype.NamedType(
        "subjectPublicKeyInfo",
        SubjectPublicKeyInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
        ),
    ),
    namedtype.OptionalNamedType(
        "exts",
        Extensions().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
        ),
    ),
)


class TrustAnchorChangeInfo(univ.Sequence):
    pass


TrustAnchorChangeInfo.componentType = namedtype.NamedTypes(
    namedtype.NamedType("pubKey", SubjectPublicKeyInfo()),
    namedtype.OptionalNamedType("keyId", KeyIdentifier()),
    namedtype.OptionalNamedType("taTitle", TrustAnchorTitle()),
    namedtype.OptionalNamedType("certPath", CertPathControls()),
    namedtype.OptionalNamedType(
        "exts",
        Extensions().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class TrustAnchorChangeInfoChoice(univ.Choice):
    pass


TrustAnchorChangeInfoChoice.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "tbsCertChange",
        TBSCertificateChangeInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
        ),
    ),
    namedtype.NamedType(
        "taChange",
        TrustAnchorChangeInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class TrustAnchorUpdate(univ.Choice):
    pass


TrustAnchorUpdate.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "add",
        TrustAnchorChoice().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.NamedType(
        "remove",
        SubjectPublicKeyInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
    namedtype.NamedType(
        "change",
        TrustAnchorChangeInfoChoice().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 3)
        ),
    ),
)


class TAMPUpdate(univ.Sequence):
    pass


TAMPUpdate.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.DefaultedNamedType(
        "terse",
        TerseOrVerbose()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
        .subtype(value="verbose"),
    ),
    namedtype.NamedType("msgRef", TAMPMsgRef()),
    namedtype.NamedType(
        "updates",
        univ.SequenceOf(componentType=TrustAnchorUpdate()).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
        ),
    ),
    namedtype.OptionalNamedType(
        "tampSeqNumbers",
        TAMPSequenceNumbers().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


tamp_update = rfc5652.ContentInfo()
tamp_update["contentType"] = id_ct_TAMP_update
tamp_update["content"] = TAMPUpdate()


# Trust Anchor Update Confirm Message

id_ct_TAMP_updateConfirm = _OID(id_tamp, 4)


class StatusCode(univ.Enumerated):
    pass


StatusCode.namedValues = namedval.NamedValues(
    ("success", 0),
    ("decodeFailure", 1),
    ("badContentInfo", 2),
    ("badSignedData", 3),
    ("badEncapContent", 4),
    ("badCertificate", 5),
    ("badSignerInfo", 6),
    ("badSignedAttrs", 7),
    ("badUnsignedAttrs", 8),
    ("missingContent", 9),
    ("noTrustAnchor", 10),
    ("notAuthorized", 11),
    ("badDigestAlgorithm", 12),
    ("badSignatureAlgorithm", 13),
    ("unsupportedKeySize", 14),
    ("unsupportedParameters", 15),
    ("signatureFailure", 16),
    ("insufficientMemory", 17),
    ("unsupportedTAMPMsgType", 18),
    ("apexTAMPAnchor", 19),
    ("improperTAAddition", 20),
    ("seqNumFailure", 21),
    ("contingencyPublicKeyDecrypt", 22),
    ("incorrectTarget", 23),
    ("communityUpdateFailed", 24),
    ("trustAnchorNotFound", 25),
    ("unsupportedTAAlgorithm", 26),
    ("unsupportedTAKeySize", 27),
    ("unsupportedContinPubKeyDecryptAlg", 28),
    ("missingSignature", 29),
    ("resourcesBusy", 30),
    ("versionNumberMismatch", 31),
    ("missingPolicySet", 32),
    ("revokedCertificate", 33),
    ("unsupportedTrustAnchorFormat", 34),
    ("improperTAChange", 35),
    ("malformed", 36),
    ("cmsError", 37),
    ("unsupportedTargetIdentifier", 38),
    ("other", 127),
)


class StatusCodeList(univ.SequenceOf):
    pass


StatusCodeList.componentType = StatusCode()
StatusCodeList.subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class TerseUpdateConfirm(StatusCodeList):
    pass


class VerboseUpdateConfirm(univ.Sequence):
    pass


VerboseUpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.NamedType("status", StatusCodeList()),
    namedtype.NamedType("taInfo", TrustAnchorChoiceList()),
    namedtype.OptionalNamedType("tampSeqNumbers", TAMPSequenceNumbers()),
    namedtype.DefaultedNamedType("usesApex", univ.Boolean().subtype(value=1)),
)


class UpdateConfirm(univ.Choice):
    pass


UpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "terseConfirm",
        TerseUpdateConfirm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "verboseConfirm",
        VerboseUpdateConfirm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class TAMPUpdateConfirm(univ.Sequence):
    pass


TAMPUpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("update", TAMPMsgRef()),
    namedtype.NamedType("confirm", UpdateConfirm()),
)


tamp_update_confirm = rfc5652.ContentInfo()
tamp_update_confirm["contentType"] = id_ct_TAMP_updateConfirm
tamp_update_confirm["content"] = TAMPUpdateConfirm()


# Apex Trust Anchor Update Message

id_ct_TAMP_apexUpdate = _OID(id_tamp, 5)


class TAMPApexUpdate(univ.Sequence):
    pass


TAMPApexUpdate.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.DefaultedNamedType(
        "terse",
        TerseOrVerbose()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
        .subtype(value="verbose"),
    ),
    namedtype.NamedType("msgRef", TAMPMsgRef()),
    namedtype.NamedType("clearTrustAnchors", univ.Boolean()),
    namedtype.NamedType("clearCommunities", univ.Boolean()),
    namedtype.OptionalNamedType("seqNumber", SeqNumber()),
    namedtype.NamedType("apexTA", TrustAnchorChoice()),
)


tamp_apex_update = rfc5652.ContentInfo()
tamp_apex_update["contentType"] = id_ct_TAMP_apexUpdate
tamp_apex_update["content"] = TAMPApexUpdate()


# Apex Trust Anchor Update Confirm Message

id_ct_TAMP_apexUpdateConfirm = _OID(id_tamp, 6)


class TerseApexUpdateConfirm(StatusCode):
    pass


class VerboseApexUpdateConfirm(univ.Sequence):
    pass


VerboseApexUpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.NamedType("status", StatusCode()),
    namedtype.NamedType("taInfo", TrustAnchorChoiceList()),
    namedtype.OptionalNamedType(
        "communities",
        CommunityIdentifierList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.OptionalNamedType(
        "tampSeqNumbers",
        TAMPSequenceNumbers().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
)


class ApexUpdateConfirm(univ.Choice):
    pass


ApexUpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "terseApexConfirm",
        TerseApexUpdateConfirm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "verboseApexConfirm",
        VerboseApexUpdateConfirm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class TAMPApexUpdateConfirm(univ.Sequence):
    pass


TAMPApexUpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("apexReplace", TAMPMsgRef()),
    namedtype.NamedType("apexConfirm", ApexUpdateConfirm()),
)


tamp_apex_update_confirm = rfc5652.ContentInfo()
tamp_apex_update_confirm["contentType"] = id_ct_TAMP_apexUpdateConfirm
tamp_apex_update_confirm["content"] = TAMPApexUpdateConfirm()


# Community Update Message

id_ct_TAMP_communityUpdate = _OID(id_tamp, 7)


class CommunityUpdates(univ.Sequence):
    pass


CommunityUpdates.componentType = namedtype.NamedTypes(
    namedtype.OptionalNamedType(
        "remove",
        CommunityIdentifierList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
        ),
    ),
    namedtype.OptionalNamedType(
        "add",
        CommunityIdentifierList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
        ),
    ),
)


class TAMPCommunityUpdate(univ.Sequence):
    pass


TAMPCommunityUpdate.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.DefaultedNamedType(
        "terse",
        TerseOrVerbose()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))
        .subtype(value="verbose"),
    ),
    namedtype.NamedType("msgRef", TAMPMsgRef()),
    namedtype.NamedType("updates", CommunityUpdates()),
)


tamp_community_update = rfc5652.ContentInfo()
tamp_community_update["contentType"] = id_ct_TAMP_communityUpdate
tamp_community_update["content"] = TAMPCommunityUpdate()


# Community Update Confirm Message

id_ct_TAMP_communityUpdateConfirm = _OID(id_tamp, 8)


class TerseCommunityConfirm(StatusCode):
    pass


class VerboseCommunityConfirm(univ.Sequence):
    pass


VerboseCommunityConfirm.componentType = namedtype.NamedTypes(
    namedtype.NamedType("status", StatusCode()),
    namedtype.OptionalNamedType("communities", CommunityIdentifierList()),
)


class CommunityConfirm(univ.Choice):
    pass


CommunityConfirm.componentType = namedtype.NamedTypes(
    namedtype.NamedType(
        "terseCommConfirm",
        TerseCommunityConfirm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
        ),
    ),
    namedtype.NamedType(
        "verboseCommConfirm",
        VerboseCommunityConfirm().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
        ),
    ),
)


class TAMPCommunityUpdateConfirm(univ.Sequence):
    pass


TAMPCommunityUpdateConfirm.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("update", TAMPMsgRef()),
    namedtype.NamedType("commConfirm", CommunityConfirm()),
)


tamp_community_update_confirm = rfc5652.ContentInfo()
tamp_community_update_confirm["contentType"] = id_ct_TAMP_communityUpdateConfirm
tamp_community_update_confirm["content"] = TAMPCommunityUpdateConfirm()


# Sequence Number Adjust Message

id_ct_TAMP_seqNumAdjust = _OID(id_tamp, 10)


class SequenceNumberAdjust(univ.Sequence):
    pass


SequenceNumberAdjust.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("msgRef", TAMPMsgRef()),
)


tamp_sequence_number_adjust = rfc5652.ContentInfo()
tamp_sequence_number_adjust["contentType"] = id_ct_TAMP_seqNumAdjust
tamp_sequence_number_adjust["content"] = SequenceNumberAdjust()


# Sequence Number Adjust Confirm Message

id_ct_TAMP_seqNumAdjustConfirm = _OID(id_tamp, 11)


class SequenceNumberAdjustConfirm(univ.Sequence):
    pass


SequenceNumberAdjustConfirm.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("adjust", TAMPMsgRef()),
    namedtype.NamedType("status", StatusCode()),
)


tamp_sequence_number_adjust_confirm = rfc5652.ContentInfo()
tamp_sequence_number_adjust_confirm["contentType"] = id_ct_TAMP_seqNumAdjustConfirm
tamp_sequence_number_adjust_confirm["content"] = SequenceNumberAdjustConfirm()


# TAMP Error Message

id_ct_TAMP_error = _OID(id_tamp, 9)


class TAMPError(univ.Sequence):
    pass


TAMPError.componentType = namedtype.NamedTypes(
    namedtype.DefaultedNamedType(
        "version",
        TAMPVersion()
        .subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))
        .subtype(value="v2"),
    ),
    namedtype.NamedType("msgType", univ.ObjectIdentifier()),
    namedtype.NamedType("status", StatusCode()),
    namedtype.OptionalNamedType("msgRef", TAMPMsgRef()),
)


tamp_error = rfc5652.ContentInfo()
tamp_error["contentType"] = id_ct_TAMP_error
tamp_error["content"] = TAMPError()


# Object Identifier Arc for Attributes

id_attributes = univ.ObjectIdentifier("2.16.840.1.101.2.1.5")


# contingency-public-key-decrypt-key unsigned attribute

id_aa_TAMP_contingencyPublicKeyDecryptKey = _OID(id_attributes, 63)


class PlaintextSymmetricKey(univ.OctetString):
    pass


contingency_public_key_decrypt_key = Attribute()
contingency_public_key_decrypt_key["type"] = id_aa_TAMP_contingencyPublicKeyDecryptKey
contingency_public_key_decrypt_key["values"][0] = PlaintextSymmetricKey()


# id-pe-wrappedApexContinKey extension

id_pe_wrappedApexContinKey = univ.ObjectIdentifier("1.3.6.1.5.5.7.1.20")


class ApexContingencyKey(univ.Sequence):
    pass


ApexContingencyKey.componentType = namedtype.NamedTypes(
    namedtype.NamedType("wrapAlgorithm", AlgorithmIdentifier()),
    namedtype.NamedType("wrappedContinPubKey", univ.OctetString()),
)


wrappedApexContinKey = Extension()
wrappedApexContinKey["extnID"] = id_pe_wrappedApexContinKey
wrappedApexContinKey["critical"] = 0
wrappedApexContinKey["extnValue"] = univ.OctetString()


# Add to the map of CMS Content Type OIDs to Content Types in
# rfc5652.py

_cmsContentTypesMapUpdate = {
    id_ct_TAMP_statusQuery: TAMPStatusQuery(),
    id_ct_TAMP_statusResponse: TAMPStatusResponse(),
    id_ct_TAMP_update: TAMPUpdate(),
    id_ct_TAMP_updateConfirm: TAMPUpdateConfirm(),
    id_ct_TAMP_apexUpdate: TAMPApexUpdate(),
    id_ct_TAMP_apexUpdateConfirm: TAMPApexUpdateConfirm(),
    id_ct_TAMP_communityUpdate: TAMPCommunityUpdate(),
    id_ct_TAMP_communityUpdateConfirm: TAMPCommunityUpdateConfirm(),
    id_ct_TAMP_seqNumAdjust: SequenceNumberAdjust(),
    id_ct_TAMP_seqNumAdjustConfirm: SequenceNumberAdjustConfirm(),
    id_ct_TAMP_error: TAMPError(),
}

rfc5652.cmsContentTypesMap.update(_cmsContentTypesMapUpdate)


# Add to the map of CMS Attribute OIDs to Attribute Values in
# rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_TAMP_contingencyPublicKeyDecryptKey: PlaintextSymmetricKey(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)


# Add to the map of Certificate Extension OIDs to Extensions in
# rfc5280.py

_certificateExtensionsMap = {
    id_pe_wrappedApexContinKey: ApexContingencyKey(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMap)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import char, namedtype, univ

from pyasn1_modules import pem, rfc5652, rfc6402


class BackwardCompatibilityTestCase(unittest.TestCase):
    pem_text = """\
MIIEJQYJKoZIhvcNAQcCoIIEFjCCBBICAQMxCzAJBgUrDgMCGgUAMIIDAgYIKwYBBQUHDAKgggL0
BIIC8DCCAuwweDB2AgECBgorBgEEAYI3CgoBMWUwYwIBADADAgEBMVkwVwYJKwYBBAGCNxUUMUow
SAIBBQwZcGl0dWNoYTEuZW1lYS5ocHFjb3JwLm5ldAwMRU1FQVxwaXR1Y2hhDBpDTUNSZXFHZW5l
cmF0b3IudnNob3N0LmV4ZTCCAmqgggJmAgEBMIICXzCCAcgCAQAwADCBnzANBgkqhkiG9w0BAQEF
AAOBjQAwgYkCgYEA0jm7SSSm2wyEAzuNKtFZFJKo91SrJq9wQwEhEKHDavZwMQOm1rZ2PF8NWCEb
PqrhToQ7rtiGLSZa4dF4bzgmBqQ9aoSfEX4jISt31Vy+skHidXjHHpbsjT24NPhrZgANivL7CxD6
Ft+s7qS1gL4HRm2twQkqSwOLrE/q2QeXl2UCAwEAAaCCAR0wGgYKKwYBBAGCNw0CAzEMFgo2LjIu
OTIwMC4yMD4GCSqGSIb3DQEJDjExMC8wHQYDVR0OBBYEFMW2skn88gxhONWZQA4sWGBDb68yMA4G
A1UdDwEB/wQEAwIHgDBXBgkrBgEEAYI3FRQxSjBIAgEFDBlwaXR1Y2hhMS5lbWVhLmhwcWNvcnAu
bmV0DAxFTUVBXHBpdHVjaGEMGkNNQ1JlcUdlbmVyYXRvci52c2hvc3QuZXhlMGYGCisGAQQBgjcN
AgIxWDBWAgECHk4ATQBpAGMAcgBvAHMAbwBmAHQAIABTAHQAcgBvAG4AZwAgAEMAcgB5AHAAdABv
AGcAcgBhAHAAaABpAGMAIABQAHIAbwB2AGkAZABlAHIDAQAwDQYJKoZIhvcNAQEFBQADgYEAJZlu
mxjtCxSOQi27jsVdd3y8NSIlzNv0b3LqmzvAly6L+CstXcnuG2MPQqPH9R7tbJonGUniBQO9sQ7C
KhYWj2gfhiEkSID82lV5chINVUFKoUlSiEhWr0tPGgvOaqdsKQcrHfzrsBbFkhDqrFSVy7Yivbnh
qYszKrOjJKiiCPMwADAAMYH5MIH2AgEDgBTFtrJJ/PIMYTjVmUAOLFhgQ2+vMjAJBgUrDgMCGgUA
oD4wFwYJKoZIhvcNAQkDMQoGCCsGAQUFBwwCMCMGCSqGSIb3DQEJBDEWBBTFTkK/OifaFjwqHiJu
xM7qXcg/VzANBgkqhkiG9w0BAQEFAASBgKfC6jOi1Wgy4xxDCQVK9+e5tktL8wE/j2cb9JSqq+aU
5UxEgXEw7q7BoYZCAzcxMRriGzakXr8aXHcgkRJ7XcFvLPUjpmGg9SOZ2sGW4zQdWAwImN/i8loc
xicQmJP+VoMHo/ZpjFY9fYCjNZUArgKsEwK/s+p9yrVVeB1Nf8Mn
"""

    def testDerCodec(self):
        layers = {}
        layers.update(rfc5652.cmsContentTypesMap)

        getNextLayer = {
            rfc5652.id_ct_contentInfo: lambda x: x["contentType"],
            rfc5652.id_signedData: lambda x: x["encapContentInfo"]["eContentType"],
            rfc6402.id_cct_PKIData: lambda x: None,
        }

        getNextSubstrate = {
            rfc5652.id_ct_contentInfo: lambda x: x["content"],
            rfc5652.id_signedData: lambda x: x["encapContentInfo"]["eContent"],
            rfc6402.id_cct_PKIData: lambda x: None,
        }

        substrate = pem.readBase64fromText(self.pem_text)

        next_layer = rfc5652.id_ct_contentInfo
        while next_layer:
            asn1Object, rest = der_decoder(substrate, asn1Spec=layers[next_layer])
            self.assertFalse(rest)
            self.assertTrue(asn1Object.prettyPrint())
            self.assertEqual(substrate, der_encoder(asn1Object))

            substrate = getNextSubstrate[next_layer](asn1Object)
            next_layer = getNextLayer[next_layer](asn1Object)

    def testOpenTypes(self):
        class ClientInformation(univ.Sequence):
            pass

        ClientInformation.componentType = namedtype.NamedTypes(
            namedtype.NamedType("clientId", univ.Integer()),
            namedtype.NamedType("MachineName", char.UTF8String()),
            namedtype.NamedType("UserName", char.UTF8String()),
            namedtype.NamedType("ProcessName", char.UTF8String()),
        )

        class EnrollmentCSP(univ.Sequence):
            pass

        EnrollmentCSP.componentType = namedtype.NamedTypes(
            namedtype.NamedType("KeySpec", univ.Integer()),
            namedtype.NamedType("Name", char.BMPString()),
            namedtype.NamedType("Signature", univ.BitString()),
        )

        openTypeMap = {
            # attributes
            univ.ObjectIdentifier("1.3.6.1.4.1.311.13.2.3"): char.IA5String(),
            univ.ObjectIdentifier("1.3.6.1.4.1.311.13.2.2"): EnrollmentCSP(),
            univ.ObjectIdentifier("1.3.6.1.4.1.311.21.20"): ClientInformation(),
            # algorithm identifier parameters
            univ.ObjectIdentifier("1.2.840.113549.1.1.1"): univ.Null(""),
            univ.ObjectIdentifier("1.2.840.113549.1.1.5"): univ.Null(""),
            univ.ObjectIdentifier("1.2.840.113549.1.1.11"): univ.Null(""),
        }

        openTypeMap.update(rfc5652.cmsAttributesMap)
        openTypeMap.update(rfc6402.cmcControlAttributesMap)

        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=rfc5652.ContentInfo(), decodeOpenTypes=True
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        eci = asn1Object["content"]["encapContentInfo"]

        self.assertEqual(rfc6402.id_cct_PKIData, eci["eContentType"])

        substrate = eci["eContent"]
        asn1Object, rest = der_decoder(
            substrate,
            asn1Spec=rfc6402.PKIData(),
            openTypes=openTypeMap,
            decodeOpenTypes=True,
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        for req in asn1Object["reqSequence"]:
            cr = req["tcr"]["certificationRequest"]

            sig_alg = cr["signatureAlgorithm"]

            self.assertIn(sig_alg["algorithm"], openTypeMap)
            self.assertEqual(univ.Null(""), sig_alg["parameters"])

            cri = cr["certificationRequestInfo"]
            spki_alg = cri["subjectPublicKeyInfo"]["algorithm"]

            self.assertIn(spki_alg["algorithm"], openTypeMap)
            self.assertEqual(univ.Null(""), spki_alg["parameters"])

            attrs = cr["certificationRequestInfo"]["attributes"]
            for attr in attrs:
                self.assertIn(attr["attrType"], openTypeMap)

                if attr["attrType"] == univ.ObjectIdentifier("1.3.6.1.4.1.311.13.2.3"):
                    self.assertEqual("6.2.9200.2", attr["attrValues"][0])

                else:
                    self.assertTrue(attr["attrValues"][0].hasValue())


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# SNMPv3 message syntax
#
# ASN.1 source from:
# http://www.ietf.org/rfc/rfc3414.txt
#
from pyasn1.type import constraint, namedtype, univ


class UsmSecurityParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("msgAuthoritativeEngineID", univ.OctetString()),
        namedtype.NamedType(
            "msgAuthoritativeEngineBoots",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, 2147483647)
            ),
        ),
        namedtype.NamedType(
            "msgAuthoritativeEngineTime",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(0, 2147483647)
            ),
        ),
        namedtype.NamedType(
            "msgUserName",
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(0, 32)
            ),
        ),
        namedtype.NamedType("msgAuthenticationParameters", univ.OctetString()),
        namedtype.NamedType("msgPrivacyParameters", univ.OctetString()),
    )



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Storing Validation Parameters in PKCS#8
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8479.txt
#

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc5652

id_attr_validation_parameters = univ.ObjectIdentifier("1.3.6.1.4.1.2312.18.8.1")


class ValidationParams(univ.Sequence):
    pass


ValidationParams.componentType = namedtype.NamedTypes(
    namedtype.NamedType("hashAlg", univ.ObjectIdentifier()),
    namedtype.NamedType("seed", univ.OctetString()),
)


at_validation_parameters = rfc5652.Attribute()
at_validation_parameters["attrType"] = id_attr_validation_parameters
at_validation_parameters["attrValues"][0] = ValidationParams()


# Map of Attribute Type OIDs to Attributes added to the
# ones that are in rfc5652.py

_cmsAttributesMapUpdate = {
    id_attr_validation_parameters: ValidationParams(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)
